import React, { useState, useCallback, useEffect } from 'react'; //react.production.min.js
import ReactDOM from 'react-dom'; //react-dom.production.min.js
import { csv, arc, pie, scaleBand, scaleLinear, max, format } from 'd3'; //d3.min.js v5.11.0


const width = 960;
const height = 500;
const margin = { top: 90, right: 60, bottom:45, left: 130 };
const xAxisLabelOffset = 50;


const csvUrl = 'Elements.csv'
// load csv data  
const useData = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = d => {
      d.Abundance = +d['CRC2016'];
      return d;
    };
    // slice data
    csv(csvUrl, row).then(data => {
      setData(data.slice(0, 17));
    });
  }, []);
  return data;
};


// y axis
const AxisLeft = ({ yScale, xValue }) =>
yScale.domain().map(tickValue => (
  <g className="tick">
    <text
      key={tickValue}
      style={{ textAnchor: 'end' }}
      x={-10}
      dy=".32em"
      y={yScale(tickValue) + yScale.bandwidth() / 2}
      title={tickValue}
    >
      {tickValue}

    </text>
    <title>{tickValue}: {} mg/kg</title>
  </g>
));


// x axis
const AxisBottom = ({ xScale, innerHeight, tickFormat }) =>
xScale.ticks().map(tickValue => (
  <g className="tick" key={tickValue} transform={`translate(${xScale(tickValue)},0)`}>
    <line y2={innerHeight} />
    <text style={{ textAnchor: 'middle' }} dy=".71em" y={innerHeight + 3}>
      {tickFormat(tickValue)}
    </text>
  </g>
));


// marks
const numberFormat = format('.2');
const NumberForm = PopValue => numberFormat(PopValue)

const Marks = ({
  data,
  xScale,
  yScale,
  xValue,
  yValue,
  tooltipFormat
}) =>
  data.map(d => (
    <rect
      className="mark"
      key={yValue(d)}
      x={0}
      y={yScale(yValue(d))}
      width={xScale(xValue(d))}
      height={yScale.bandwidth()}
    >
    <title>{d.Element + ' (' + d.Symbol + ') [Z=' + d.Z + ']: ' + NumberForm(xValue(d))} mg/kg</title>  
   
    </rect>
  ));


// plot data
const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const yValue = d => d.Element
  const xValue = d => d.CRC2016; // d.CRC2016 // d.Geochemistry1974

  const siFormat = format('.2');
  const xAxisTickFormat = tickValue => siFormat(tickValue);

  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .paddingInner(0.15);

  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);


  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <text
          className="title"
          x={innerWidth / 2-40}
          y={-30}
          textAnchor="middle"
        >
   Abundance of rare Elements in the Earth's crust
        </text>
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
        />
        <AxisLeft yScale={yScale}  />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset -10}
          textAnchor="middle"
        >
    Abundance [ppm = mg/kg] 
        </text>
        <Marks
          data={data}
          xScale={xScale}
          yScale={yScale}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');

ReactDOM.render(<App />, rootElement);

// //